      subroutine init_partonshower
      include 'psinclude.inc'
      double precision mass(5)
      common/imode/imode
      common/amasses/mass
! imode = 11 H --> e+e- mu+mu-    with order alpha Parton Shower
! imode = 12 H --> e+e- mu+mu-    with exponentiatied Parton Shower
! imode = 13 H --> e+e- e+e-      with order alpha Parton Shower
! imode = 14 H --> e+e- e+e-      with exponentiatied Parton Shower
! imode = 15 H --> mu+mu- mu+mu-  with order alpha Parton Shower
! imode = 16 H --> mu+mu- mu+mu-  with exponentiated Parton Shower
!
! imode = 20  Born (via Parton Shower... :-) )

      pi = 4.d0*atan(1.d0)
c      alpha = 1.d0/137.03599911d0
     
      alphaps = alpha*rescaleQED
      

      ch1 = 1d0
      ch2 = 1d0
      ch3 = 1d0
      ch4 = 1d0

      do k = 1,4
         charge_factor(k) = 1
      enddo

      ptqed = 'yes'
      
!      print*,'Parton Shower at order alpha or exp.? (alpha/exp/no)'
!      read*,qedorder

      qedorder = 'no' ! imode > 20....
      if (imode.eq.12.or.imode.eq.14.or.imode.eq.16) qedorder = 'exp'
      if (imode.eq.11.or.imode.eq.13.or.imode.eq.15) qedorder = 'alpha'

      if (qedorder.eq.'no') then
         do k = 1,4
            charge_factor(k) = 0
         enddo
         qedorder = 'exp'
      endif

      m1 = mass(1)
      m2 = mass(2)
      m3 = mass(3)
      m4 = mass(4)

      eps = 5.d-3
      if (qedorder.eq.'exp') eps = 5d-6 !! default eps = 5.d-6

      iplus  = -2.d0*log(eps)-1.5d0+2.d0*eps-0.5d0*eps**2
      return
      end

*********** subroutines for QED Parton Shower radiation ***********
      subroutine QEDPartonShower(q21,q22,p1,p2,p3,p4,qph,qpen,ie)
      include 'psinclude.inc'
      dimension qpen(0:3)
      data icntikine /0/
      dimension x(40),xvect(10)
*               ^^^^ sono le frazioni di energia rimanenti ai fotoni
      ie = 0

      irigenera = 1
      do while(irigenera.eq.1)
         if (qedorder.eq.'exp') then
            
            tmp = charge_factor(1)*ch1
            call dqed(q21,m1,tmp,xvect)
            do k = 1, 10
               x(k) = xvect(k)
            enddo
            
            tmp = charge_factor(2)*ch2
            call dqed(q21,m2,tmp,xvect)  
            do k = 1, 10
               x(k+10) = xvect(k)
            enddo
            
            tmp = charge_factor(3)*ch3
            call dqed(q22,m3,tmp,xvect)  
            do k = 1, 10
               x(k+20) = xvect(k)
            enddo
            
            tmp = charge_factor(4)*ch4
            call dqed(q22,m4,tmp,xvect)  
            do k = 1, 10
               x(k+30) = xvect(k)
            enddo      
         else   ! if (qedorder.eq.'alpha')
            call dqed_oalpha(q21,q22,m1,m2,m3,m4,ch1,ch2,ch3,
     >           ch4,x)
         endif
*     
         if (ptqed.eq.'no') then
            call kinematics_nopt(x,p1,p2,p3,p4,qph,iekine)
         else
            call kinematics_pt(x,p1,p2,p3,p4,qph,iekine)
         endif
         if (iekine.eq.0) irigenera = 0
      enddo

!      call compact_phmom(qph)
      call fotonepiuenergetico(qph,qpen)
      
      return
      end
*
*************************************************************************
      subroutine dqed_oalpha(q21,q22,ama,amb,amc,amd,
     .     car1,car2,car3,car4,x)
* QED Parton Shower at order alpha
      include 'psinclude.inc'
      dimension x(40)
      double precision mqua(4),qqua(4)
      double precision rvec(2)
      
      do k=1,40
         x(k) = 0.d0
      enddo      
      qqua(1) = (charge_factor(1)*car1)**2
      qqua(2) = (charge_factor(2)*car2)**2
      qqua(3) = (charge_factor(3)*car3)**2
      qqua(4) = (charge_factor(4)*car4)**2
     
      qsum = qqua(1) + qqua(2) + qqua(3) + qqua(4)
      if (qsum.lt.1.d-1) return

      mqua(1) = ama**2*exp(1.d0)
      mqua(2) = amb**2*exp(1.d0)
      mqua(3) = amc**2*exp(1.d0)
      mqua(4) = amd**2*exp(1.d0)
      
      pnb = ffsoa(q21,q22,mqua,qqua)

      if (pnb.lt.0.d0) then
         print*,' WARNING: Sudakov Form Factor < 0 !',pnb
      endif
      
      call wraprnd(rvec,2)      
      r1 = 1.d0*rvec(1)
      r2 = 1.d0*rvec(2)
      
      if (r1.lt.pnb) return
      
      leg   = 1
      pnorm = (1.d0-pnb)*2.d0*pi/alphaps/iplus
      pirr  = qqua(leg)*log(q21/mqua(leg))/pnorm
      do while(r2.gt.pirr)
         leg  = leg  + 1
         if (leg.le.2) al   = log(q21/mqua(leg))
         if (leg.gt.2) al   = log(q22/mqua(leg))         
         pirr = pirr + qqua(leg)*al/pnorm
      enddo
      
      call ap_vertex(y)
      y = 1.d0 - y 
      
      index    = 10*(leg-1)+1
      x(index) = y
      return
      end
*******************************************************      
      function ffsoa(q21,q22,mqa,qqa)
* Sudakov pippo factor at order alpha
      include 'psinclude.inc'
      double precision mqa(4),qqa(4)
      ffsoa = 0.d0
      do i = 1,4
         if (i.le.2) al = log(q21/mqa(i))
         if (i.gt.2) al = log(q22/mqa(i))
         ffsoa = ffsoa + qqa(i)*al
      enddo      
      ffsoa = ffsoa*alphaps/2.d0/pi*iplus      
      ffsoa = 1.d0 - ffsoa      
      return
      end
*************************************************************************
      subroutine dqed(q2,m,ch,xv)
* Exponentiated Parton Shower
*  q2 is the SF Q^2 scale, m and ch are the mass and of the emitting particle,
*  xv(10) are the photons energy fractions extracted by the PS
      include 'psinclude.inc'
      common/n_phot/nphot
      double precision r(1)
      double precision m
      dimension xv(10)
*      
      do k = 1, 10
         xv(k) = 0.d0
      enddo
      nphot = 0
*
      if (abs(ch).lt.1.d-1) return
*
      q2min = m**2*exp(1.d0)

      ak2   = q2min
      irigenera = 1
      do while(irigenera.eq.1)
         call wraprnd(r,1)
*
         rand = 1.d0*r(1)
         ffsn = ffs(q2,ak2,ch)
       
         if (rand.le.ffsn) return
*
         nphot = nphot + 1
*     
         akp2 = -2.d0*pi/(ch**2*alphaps)/iplus*log(rand)
         akp2 = ak2*exp(akp2)                  
*     
         call ap_vertex(y)
*     
         if(nphot.le.10) then 
            xv(nphot) = 1.d0-y
         endif
*     
         ak2 = akp2
      enddo 
      return
      end
************************************************************************
      function ffs(q2,qp2,c)
* no-branching probability (Sudakov pippo factor) 
      include 'psinclude.inc'
      ffs = -c**2*alphaps/2.d0/pi*log(q2/qp2)*iplus
      ffs = exp(ffs)
      return
      end
************************************************************************
      subroutine ap_vertex(x)
*  x generation according to ap splitting function  
*  (1+x^2)/(1-x), 0 <= x <= 1-eps
      include 'psinclude.inc'
      double precision r(2)
*      
      alne = log(eps)
      irigenera = 1
      do while(irigenera.eq.1)
         call wraprnd(r,2)
         cx = r(1)
         omx = exp(cx*alne)
         x = 1.d0 - omx
         rx = r(2)*2.d0/omx
         px = (1.d0 + x**2)/omx
         if (rx.lt.px) irigenera = 0
      enddo
      return
      end
********************************
      subroutine kinematics_nopt(x,p1,p2,p3,p4,qph,ie)
* subroutine for collinear photons generation. Photons are in qph(40,0-3)
      include 'psinclude.inc'
      dimension x(40)
      ie = 0
      do k = 1,40
         qph(k,0) = 0.d0
         qph(k,1) = 0.d0
         qph(k,2) = 0.d0
         qph(k,3) = 0.d0
      enddo
      xctrl = x(1)+x(11)+x(21)+x(31)
      if (xctrl.lt.1.d-10) return

! for the moment, photons are strictly collinear to the fermions
      o1 = p1(0)
      o2 = p2(0)
      o3 = p3(0)
      o4 = p4(0)
      do k = 1,10
         qph(k,0)    = o1*x(k)
         qph(k+10,0) = o2*x(k+10)
         qph(k+20,0) = o3*x(k+20)
         qph(k+30,0) = o4*x(k+30)
         o1 = o1 - qph(k,0)
         o2 = o2 - qph(k+10,0)
         o3 = o3 - qph(k+20,0)
         o4 = o4 - qph(k+30,0)
      enddo

      p1mod = sqrt(p1(1)**2 + p1(2)**2 + p1(3)**2)
      p2mod = sqrt(p2(1)**2 + p2(2)**2 + p2(3)**2)
      p3mod = sqrt(p3(1)**2 + p3(2)**2 + p3(3)**2)
      p4mod = sqrt(p4(1)**2 + p4(2)**2 + p4(3)**2)

      p1(0) = o1
      p2(0) = o2
      p3(0) = o3
      p4(0) = o4
      do k = 1,3
         p1(k) = p1(0)*p1(k)/p1mod
         p2(k) = p2(0)*p2(k)/p2mod
         p3(k) = p3(0)*p3(k)/p3mod
         p4(k) = p4(0)*p4(k)/p4mod
      enddo

      do k = 1,40
         qph(k,3) = qph(k,0)
      enddo
*
      do k = 1,10
         if (qph(k,0).gt.0.d0) then
            do i=0,3
               ptmp(i) = qph(k,i)
            enddo
            call rot(-1,p1,ptmp,ptmp)
            do i=0,3
               qph(k,i) = ptmp(i)
            enddo
         endif
      enddo
      do k = 11,20
         if (qph(k,0).gt.0.d0) then
            do i=0,3
               ptmp(i) = qph(k,i)
            enddo
            call rot(-1,p2,ptmp,ptmp)
            do i=0,3
               qph(k,i) = ptmp(i)
            enddo
         endif
      enddo
      do k = 21,30
         if (qph(k,0).gt.0.d0) then
            do i=0,3
               ptmp(i) = qph(k,i)
            enddo
            call rot(-1,p3,ptmp,ptmp)
            do i=0,3
               qph(k,i) = ptmp(i)
            enddo
         endif
      enddo
      do k = 31,40
         if (qph(k,0).gt.0.d0) then
            do i=0,3
               ptmp(i) = qph(k,i)
            enddo
            call rot(-1,p4,ptmp,ptmp)
            do i=0,3
               qph(k,i) = ptmp(i)
            enddo
         endif
      enddo
*
      return
      end
******************************************************************
      subroutine kinematics_pt(x,p1,p2,p3,p4,qph,ie)
* subroutine for photons generation with pt. Photons are in qph(40,0-3)
      include 'psinclude.inc'
      common/ichansam/ichannelsampling
      common/imode/imode

      dimension x(40),vers(0:3)
      double precision kone1(0:3),kone2(0:3)

      ie = 0
      do k = 1,40
         do i = 0,3
            qph(k,i) = 0.d0
         enddo
      enddo
      xctrl = x(1)+x(11)+x(21)+x(31)
      if (xctrl.lt.1.d-10) return
*
      E1 = p1(0)
      E2 = p2(0)
      E3 = p3(0)
      E4 = p4(0)
      
      irigenera = 1
      do while(irigenera.eq.1)
         do k = 0,3
            kone1(k) = 0.d0
            kone2(k) = 0.d0
         enddo
         
** Generation of photons....
         o1 = E1
         o2 = E2
         o3 = E3
         o4 = E4
         do k = 1,10
            qph(k,0)    = o1*x(k)
            qph(k+10,0) = o2*x(k+10)
            qph(k+20,0) = o3*x(k+20)
            qph(k+30,0) = o4*x(k+30)
*
            if (qph(k,0).gt.0.d0) then            
               call alongp(p1,vers,m1,anorm,w)
               do i = 1,3
                  qph(k,i) = qph(k,0)*vers(i)
               enddo
            endif
            if (qph(k+10,0).gt.0.d0) then            
               call alongp(p2,vers,m2,anorm,w)
               do i = 1,3
                  qph(k+10,i) = qph(k+10,0)*vers(i)
               enddo
            endif
            if (qph(k+20,0).gt.0.d0) then            
               call alongp(p3,vers,m3,anorm,w)
               do i = 1,3
                  qph(k+20,i) = qph(k+20,0)*vers(i)
               enddo
            endif
            if (qph(k+30,0).gt.0.d0) then            
               call alongp(p4,vers,m4,anorm,w)
               do i = 1,3
                  qph(k+30,i) = qph(k+30,0)*vers(i)
               enddo
            endif
*     
            o1 = o1 - qph(k,0)
            o2 = o2 - qph(k+10,0)
            o3 = o3 - qph(k+20,0)
            o4 = o4 - qph(k+30,0)
         enddo
         ic1 = charge_factor(1)
         ic2 = charge_factor(2)
         ic3 = charge_factor(3)
         ic4 = charge_factor(4)

! ichannelsampling is set in the phasespace4f routine!!!         
! if particles are not identical, the switch is not needed
         if (imode.eq.11.or.imode.eq.12.or.imode.eq.21)
     >        ichannelsampling =1

         if (ichannelsampling.eq.1) then         
            do k = 1,10
               do i = 0,3
                  kone1(i) = kone1(i) + qph(k,i)    + qph(k+10,i)
                  kone2(i) = kone2(i) + qph(k+20,i) + qph(k+30,i)
               enddo
            enddo
            call rebuild_mom(kone1,p1,p2,ic1,ic2,ie1)
            call rebuild_mom(kone2,p3,p4,ic3,ic4,ie2)    
         else
            do k = 1,10
               do i = 0,3
c???? era sbagliato nei files di Luca????
c$$$                  kone1(i) = kone1(i) + qph(k,i)    + qph(k+20,i)
c$$$                  kone2(i) = kone2(i) + qph(k+10,i) + qph(k+30,i)
c????????
                  kone1(i) = kone1(i) + qph(k,i)    + qph(k+30,i)
                  kone2(i) = kone2(i) + qph(k+10,i) + qph(k+20,i)
               enddo
            enddo
            call rebuild_mom(kone1,p1,p3,ic1,ic3,ie1)
            call rebuild_mom(kone2,p2,p4,ic2,ic4,ie2)
         endif
         
         ie = ie1 + ie2
         if (ie.gt.0) ie = 1
         irigenera=0 ! never regenerated here!
      enddo
      return
      end
*************************************************************
         subroutine rebuild_mom(kone,p1,p2,ic1,ic2,ie)
* p1,p2 and kone are initially not related. At the end,
* we will have p1+p2+kone = (p1+p2 at the beginning)
         include 'psinclude.inc'
         double precision r(1)
         double precision kone(0:3)
         dimension vboost1(0:3),psum(0:3),vboost2(0:3)
* I prefer to have local copy of the variables...
         dimension bigk(0:3),pa(0:3),pb(0:3),vers(0:3)
*
         ie = 0
         if (kone(0).le.1.d-10) return
         do k = 0,3
            pa(k) = p1(k)
            pb(k) = p2(k)
            bigk(k) = kone(k)
            psum(k) = pa(k) + pb(k)
            vboost1(k) = 0.d0
            vboost2(k) = 0.d0
         enddo

** when both particles are charged, the "reference" particle is simmetrized
         iswap = 0
         if (ic1.eq.ic2) then
            call wraprnd(r,1)
            if (r(1).lt.0.5) iswap = 1
         endif
         if (iswap.eq.1) call exchange_mom(pa,pb)

* boost where psum is at rest         
         vboost1(1) = psum(1)/psum(0)
         vboost1(2) = psum(2)/psum(0)
         vboost1(3) = psum(3)/psum(0)
         bcm1=sqrt(vboost1(1)**2+vboost1(2)**2+vboost1(3)**2)
         gcm1=1.d0/sqrt(1.d0-bcm1**2)         

         call boost(gcm1,vboost1,psum,psum)
         call boost(gcm1,vboost1,pa,pa) 
         call boost(gcm1,vboost1,pb,pb)
         call boost(gcm1,vboost1,bigk,bigk)

         pamod = sqrt(tridot(pa,pa))
         vers(0) = 1.d0
         do k=1,3
            vers(k) = pa(k)/pamod            
         enddo

! Now I boost where psum - bigk is at rest
         do k=0,3
            psum(k) = psum(k) - bigk(k)
         enddo

         ctrl = dot(psum,psum)
         if (ctrl.lt.0.d0) then
            ie = 1
            return
         endif

         vboost2(1) = psum(1)/psum(0)
         vboost2(2) = psum(2)/psum(0)
         vboost2(3) = psum(3)/psum(0)

         bcm2=sqrt(vboost2(1)**2+vboost2(2)**2+vboost2(3)**2)
         gcm2=1.d0/sqrt(1.d0-bcm2**2)         

         call boost(gcm2,vboost2,psum,psum)
         call boost(gcm2,vboost2,pa,pa) 
         call boost(gcm2,vboost2,pb,pb)
         call boost(gcm2,vboost2,vers,vers)

! now I recalculate pa and pb back to back
         vmod = sqrt(tridot(vers,vers))
         pa(0) = psum(0)/2.d0
         pa(1) = pa(0)*vers(1)/vmod
         pa(2) = pa(0)*vers(2)/vmod
         pa(3) = pa(0)*vers(3)/vmod
         pb(0) = pa(0)
         pb(1) = -pa(1)
         pb(2) = -pa(2)
         pb(3) = -pa(3)

! ...and now back in the original frame
         do k = 1,3
            vboost1(k) = -vboost1(k)
            vboost2(k) = -vboost2(k)
         enddo
         call boost(gcm2,vboost2,pa,pa) 
         call boost(gcm2,vboost2,pb,pb)
         call boost(gcm1,vboost1,pa,pa) 
         call boost(gcm1,vboost1,pb,pb)

         if (iswap.eq.1) call exchange_mom(pa,pb)

         do k=0,3
            p1(k) = pa(k)
            p2(k) = pb(k)
         enddo
         return
         end
*************************************************************
      subroutine alongp(p,vers,am,anorm,w)
      implicit double precision (a-h,o-z)
      dimension p(0:3),vers(0:3),ptmp(0:3)
      double precision rv(1)

      b1 = (1.d0 - am/p(0))*(1.d0 + am/p(0))

      vers(0) = 1.d0
      if (b1.lt.0.d0) then ! (=> p(0) < am ....)
         vers(1) = 0.d0
         vers(2) = 0.d0
         vers(3) = vers(0)
         call rot(-1,p,vers,vers)
         w = 1.d0
         anorm = 1.d0
         return
      endif

      b1 = sqrt(b1)

      call wraprnd(rv,1)
      phph = 2.d0*pi*rv(1)
      call wraprnd(rv,1)

      an  = -1.d0/b1*log((1.d0-b1)/(1.d0+b1))
      cth = (1.d0 - (1.d0+b1)*
     #     exp(-b1*an*rv(1)))/b1
      
      sth = sqrt(1.d0-cth*cth)
      cph = cos(phph)
      sph = sin(phph)
      
      vers(1) = vers(0)*sph*sth
      vers(2) = vers(0)*cph*sth
      vers(3) = vers(0)*cth
                  
      call rot(-1,p,vers,vers)
      
      call alongp_nandw(p,vers,am,anorm,w)

      return
      end
********************
      subroutine alongp_nandw(p,vers,am,anorm,w)
      implicit double precision (a-h,o-z)
      dimension p(0:3),vers(0:3)

      b1 = (1.d0 - am/p(0))*(1.d0 + am/p(0))
      b1 = sqrt(b1)
      anorm  = -1.d0/b1*log((1.d0-b1)/(1.d0+b1))
      w = p(0)*vers(0)/dot(p,vers)/anorm
! w e' normalizzato a 1!
      return
      end
***************************
*************************************************
      subroutine duefotonipiuenergetici(q,q1,q2)
      implicit double precision (a-h,o-z)
      dimension q(40,0:3),q1(0:3),q2(0:3)
* 
*  leading energetic photon is extracted
*
        do i = 0,3
           q1(i) = 0.d0
           q2(i) = 0.d0
        enddo

        sum = 0.d0
	do k=1,40
          sum = sum + q(k,0)
        enddo
        if (sum.lt.1.d-8) return

        enphot = q(1,0)
        enhard = enphot
        j=1
* 
        do i = 1,40
           enphot = q(i,0)
           if (enhard.ge.enphot) then
              enhard = enhard
           else
              j = i
              enhard = enphot
           endif
        enddo 
*   
        enhardl = enhard
        jl = j
*
        if (j.gt.0) then
           do i = 0,3
              q1(i) = q(j,i)
           enddo
        endif
*  next-to-leading energetic photon is extracted
      enhard = 0.d0
      j = 0
      do i = 1,40
         if (i.ne.jl) then
            enphot = q(i,0)
            if (enhard.ge.enphot) then
               enhard = enhard
            else
               j = i
               enhard = enphot
            endif
         endif
      enddo 
*   
      enhard = enhard
      if (j.gt.0) then
         do i = 0,3
            q2(i) = q(j,i)
         enddo
      endif
      return
      end
********************
      subroutine fotonepiuenergetico(q,q1)
      implicit double precision (a-h,o-z)
      dimension q(40,0:3),q1(0:3)
* 
*  leading energetic photon is extracted
*
      do i = 0,3
         q1(i) = 0.d0
      enddo
      
      sum = 0.d0
      do k=1,40
         sum = sum + q(k,0)
      enddo
      if (sum.lt.1.d-8) return
      
      enphot = q(1,0)
      enhard = enphot
      j=1
*     
      do i = 1,40
         enphot = q(i,0)
         if (enphot.gt.0.d0) then
            if (enhard.ge.enphot) then
               enhard = enhard
            else
               j = i
               enhard = enphot
            endif
         endif
      enddo
*     
      enhardl = enhard
      jl = j
*     
      if (j.gt.0) then
         do i = 0,3
            q1(i) = q(j,i)
         enddo
      endif        
      return
      end
